import { MDynamicDialogProperties } from '../models/m-dynamic-dialog-properties';
import { Injector, Injectable, ComponentFactoryResolver, ApplicationRef, EmbeddedViewRef, Type } from '@angular/core'; 
import { MDynamicDialogParameter } from '../models/m-dynamic-dialog-parameter';
import { MDynamicDialogReference } from './m-dynamic-dialog-reference.service';
import { MDynamicDialogData } from './m-dynamic-dialog-data.service';
import { MDynamicDialogComponent } from '../m-dynamic-dialog/m-dynamic-dialog.component';

@Injectable({
  providedIn: "root"
})

export class MDynamicDialogService {

  constructor(
    private componentFactoryResolver: ComponentFactoryResolver,
    private applicationRef: ApplicationRef,
    private injector: Injector,
  ) { }

  createAndGetDialogReferenceInstance(parameter: MDynamicDialogParameter) {
 

    let reference = new MDynamicDialogReference();
    this.generateBaseDialog(reference, parameter);

    return reference;
  }

  checkIfDialogContExist() {
    var container = document.getElementById("m-dynamic-dialog-container")
    return container != null;
  }

  generateBaseDialog(reference: MDynamicDialogReference, parameter: MDynamicDialogParameter) {

    if (!this.checkIfDialogContExist()){
      this.generateDialogContainer();
    }

    var injectorCustom = Injector.create(
      [
        {
          provide: MDynamicDialogReference,
          useValue: reference
        }
      ]
    )

    var dialogFactory = this.componentFactoryResolver.resolveComponentFactory(MDynamicDialogComponent).create(injectorCustom);
    dialogFactory.instance.setComponentBuilders(this.componentFactoryResolver); 
    dialogFactory.instance.parameter = parameter;
    dialogFactory.instance.MDynamicDialogData = this.generateDialogData(parameter.Properties.data);


    this.applicationRef.attachView(dialogFactory.hostView);

    const dialog = (dialogFactory.hostView as EmbeddedViewRef<any>).rootNodes[0] as HTMLElement;
    dialog.id = reference.dialogId;

    let dialogContainer = document.getElementById("m-dynamic-dialog-container");
    dialogContainer.append(dialog)
  }

  generateDialogContainer() {
    var dialogContainer = document.createElement("div");
    dialogContainer.id = "m-dynamic-dialog-container";
    dialogContainer.style.height = "100%";
    dialogContainer.style.width = "100%";
    dialogContainer.style.position = "absolute !important";
    dialogContainer.style.background = "background: rgba(0, 0, 0, 0.5); !important"
    
    document.body.insertBefore(dialogContainer, document.body.firstChild);
  }

  generateDialogData(data: any) {
    let dialogData = new MDynamicDialogData();
    dialogData.data = data;

    return dialogData;
  }

  open(component: Type<any>, properties: MDynamicDialogProperties) {
    let parameter = new MDynamicDialogParameter();
    parameter.Component = component;
    parameter.Properties = properties;
    return this.createAndGetDialogReferenceInstance(parameter);
  }
}