import { Component, ComponentFactoryResolver, ViewChild, Injector } from '@angular/core';
import { MDynamicDialogData } from '../m-dynamic-dialog-services/m-dynamic-dialog-data.service';
import { MDynamicDialogReference } from '../m-dynamic-dialog-services/m-dynamic-dialog-reference.service';
import { MDynamicDialogParameter } from '../models/m-dynamic-dialog-parameter';
import { MDynamicDialogBodyComponent } from './m-dynamic-dialog-body/m-dynamic-dialog-body.component';
import { MDynamicDialogProperties } from '../models/m-dynamic-dialog-properties';

@Component({
  selector: 'm-dynamic',
  templateUrl: './m-dynamic-dialog.component.html',
  styleUrls: ['./m-dynamic-dialog.component.scss']
})

export class MDynamicDialogComponent {

  headerHeight = "7vh";
  footerHeight = "5vh";
  bodyHeight: string = "";

  loaded: boolean = false;
  
  parameter: MDynamicDialogParameter;

  id: string = "";

  MDynamicDialogData: MDynamicDialogData;

  componentFactoryResolver: ComponentFactoryResolver = null; 

  @ViewChild(MDynamicDialogBodyComponent, { static: false }) content: any;

  constructor(
    private MDynamicDialogReference: MDynamicDialogReference
  ) { }

  ngOnInit() {
    this.calculateDialogPropertiesAndSet(this.parameter.Properties)
  }

  ngAfterViewInit(): void {
    this.createComponent(this.componentFactoryResolver);
  }

  setComponentBuilders(cfr: ComponentFactoryResolver) {
    this.componentFactoryResolver = cfr; 
  }

  calculateDialogPropertiesAndSet(properties: MDynamicDialogProperties) {

    if(properties.headerHeight){
      this.headerHeight = properties.headerHeight
    }

    if(properties.footerHeight){
      this.footerHeight = properties.footerHeight;
    }

    //console.log(properties);
    if (properties.showHeader === true)
      this.headerHeight = "0"

    if (properties.showFooter === true)
      this.footerHeight = "0"

    this.bodyHeight = properties.height 

    this.loaded = true;

    return this;
  }

  createComponent(cfr: ComponentFactoryResolver) {
    const injector = Injector.create({
      providers: [
        {
          provide: MDynamicDialogReference,
          useValue: this.MDynamicDialogReference
        },
        {
          provide: MDynamicDialogData,
          useValue: this.MDynamicDialogData
        }
      ]
    })

    var dynamicComponent = cfr.resolveComponentFactory(this.parameter.Component);
    this.content.viewContainerRef.createComponent(dynamicComponent, 0, injector);
  }

  save() {
    this.MDynamicDialogReference.save();
  }

  close() {
    this.MDynamicDialogReference.close();
  }

  outsideClickClose() {
    this.MDynamicDialogReference.close();
  }
}
